
import java.awt.Color;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Random;
import java.util.Set;

import arbres.*;
import carte.*;
import graphe.*;

public class Test {
	static GrapheVille g;
	static Ville v1;
	static Ville v2;
	static Carte carte;

	static Ville getFirst(Collection<Ville> c) {
		for (Ville v : c)
			return v;
		return null;
	}

	public static void initMayotte(){
		Carte ens = new Carte("mf.txt");
		v1 = ens.premiereVille("Chembenioumba");
		v2 = ens.premiereVille("Boueni");
		g = new GrapheVille();
		for (Ville v : ens.villes()) {
			g.ajouterVille(v);
		}
		g.connectNeighbours(2000);
	}

	public static void initFrance(int distance){
		carte = new Carte("fr.txt");
		v1 = carte.premiereVille("Palaiseau");
		g = new GrapheEuclidien(distance);
		for (Ville v : carte.villes()) {
			g.ajouterVille(v);
		}
		g.connectNeighbours(distance);
	}

	public static boolean existeArc(GrapheVille g, Ville s, Ville t) {
		for (Arc a : g.arcsSortant(s))
			if (a.target.equals(t))
				return true;
		return false;
	}

	public static void testAjouterArc(GrapheVille g, Arc a) {
		try {
			g.ajouterArc(a);
			throw new AssertionError("an IllegalArgumentException is expected.");
		} catch (IllegalArgumentException e) {}
	}

	static final Color TRANSLUCENT_GRAY = new Color(192, 192, 192, 64);
	static final Color TRANSLUCENT_ORANGE = new Color(255, 200, 0, 128);

	public static void test1() {
		Carte ens = new Carte("mf.txt");
		GrapheVille graphe = new GrapheVille();
		Ville v1 = ens.premiereVille("Chembenioumba");
		Ville v2 = ens.premiereVille("Boueni");
		testAjouterArc(graphe, new Arc(v1, v1, 1));
		graphe.ajouterVille(v1);
		testAjouterArc(graphe, new Arc(v2, v1, 2));
		graphe.ajouterArc(new Arc(v1, v2, 3));
		testAjouterArc(graphe, new Arc(v2, v1, 2));
		graphe = new GrapheVille();
		for (Ville v : ens.villes())
			graphe.ajouterVille(v);
		System.out.println("There are " + graphe.villes().size() + " nodes in this graph.");
		int i = 0;
		v2 = null;
		for (Ville v : graphe.villes()) {
			if (++i < 10)
				System.out.println(v);
			if (v2 != null)
				graphe.ajouterArc(new Arc(v, v2, 1));
			v2 = v;
		}
		for (Ville v : ens.villes())
			graphe.ajouterVille(v); // does nothing
			int arcs = 0;
			for (Ville v : ens.villes())
				arcs += graphe.arcsSortant(v).size();
			System.out.println("There are " + arcs + " arcs in this graph.");
			Visualisation f = new Visualisation("Test 1");
			f.setOvalRadius(4);
			f.definirGraphe(graphe);
	}

	public static void test2() {
		initMayotte();
		// test de reciprocite
		for (Ville s : g.villes())
			for (Arc a : g.arcsSortant(s)) {
				if (!existeArc(g, a.target, s))
					throw new Error("l'arc " + a + " n'a pas de reciproque");
			}
		// affichage graphique
		Visualisation f = new Visualisation("Test 2");
		f.setOvalRadius(4);
		f.definirGraphe(g);
		Map<Arc, Color> couleurs = new HashMap<Arc, Color>();
		for (Ville s : g.villes())
			for (Arc a : g.arcsSortant(s))
				if (a.length < 1500)
					couleurs.put(a, Color.green);
				else couleurs.put(a, Color.red);
		f.couleurArcs(couleurs);
		int arcs = 0;
		for (Ville v : g.villes())
			arcs += g.arcsSortant(v).size();
		System.out.println("There are " + arcs + " arcs in this graph.");
	}
	
		//Q3: ArcComparator
	public static void test3(){
		carte = new Carte("fr.txt");
		Ville v0 = carte.premiereVille("Palaiseau");
		Ville v1 = carte.premiereVille( "Paris" );
		Ville v2 = carte.premiereVille( "Melun" );
		Arc a0 = new Arc(v0, v1, v0.distance(v1));
		Arc a1 = new Arc(v1, v2, v1.distance(v2));
		Arc a2 = new Arc(v0, v2, v0.distance(v2));
		ArcComparator c = new ArcComparator( );
		System.out.println( c.compare( a0, a1 ) );
		System.out.println( c.compare( a1, a2 ) );
		System.out.println( c.compare( a2, a0 ) );
		System.out.println( c.compare( a0, a0 ) );
	}
	
	  // Q4: UnionFind
    public static void test4( ){
    	carte = new Carte("fr.txt");
        UnionFind u = new UnionFind( );
        Ville v0 = carte.premiereVille( "Palaiseau" );
        Ville v1 = carte.premiereVille( "Paris" );
        Ville v2 = carte.premiereVille( "Melun" );
        Ville v3 = carte.premiereVille( "Orly" );
        Ville v4 = carte.premiereVille( "Massy" );
        Ville v5 = carte.premiereVille( "Clamart" );
        u.union( v0, v1 );
        u.union( v2, v3 );
        u.union( v4, v5 );
        u.union( v0, v3 );
        System.out.println( u.find( v0 ).equals( u.find( v2 ) ) );
        System.out.println( u.find( v0 ).equals( u.find( v1 ) ) );
        System.out.println( u.find( v3 ).equals( u.find( v5 ) ) );
    }
	
	public static void analyzeMonoColor( Collection<Arc> arcs, boolean display, String title ) {
		int length = 0;
		Set<Ville> villes = new HashSet<Ville>();
		for (Arc a : arcs) {
			length += a.length;
			villes.add(a.source);
			villes.add(a.target);
		}
		System.out.println("Total length: " + " : " + length + " m");
		System.out.println(villes.size() + " connected cities");
		if (!display)
			return;
		Map<Ville, Color> couleurVilles = new HashMap<Ville, Color>();
		Map<Arc, Color> couleurArcs = new HashMap<Arc,
				Color>();
		for (Ville v : villes)
			couleurVilles.put(v, TRANSLUCENT_ORANGE);
	
		for (Arc a : arcs)
			couleurArcs.put(a, TRANSLUCENT_ORANGE);
		Visualisation f = new Visualisation(title);
		f.couleurVilles(couleurVilles);
		f.couleurArcs(couleurArcs);
		f.definirGraphe(g);
	}
	


	public static void analyzeMultiColor( Collection<Collection<Arc>> listes, boolean display, String title ) {
		int length = 0;
		Set<Ville> villes = new HashSet<Ville>();
		for (Collection<Arc> arcs:listes){
			for (Arc a : arcs) {
				length += a.length;
				villes.add(a.source);
				villes.add(a.target);
			}
		}
		System.out.println("Total length " + " : " + length + " m");
		System.out.println(villes.size() + " connected cities");
		System.out.println(listes.size()+" components in the forest");
		if (!display)
			return;
		Map<Ville, Color> couleurVilles = new HashMap<Ville, Color>();
		Map<Arc, Color> couleurArcs = new HashMap<Arc,Color>();
		for (Ville v : villes)
			couleurVilles.put(v, TRANSLUCENT_ORANGE);
		Random rand=new Random();
		for (Collection<Arc> arcs:listes){
			Color c=new Color(rand.nextInt(256),rand.nextInt(256),rand.nextInt(256));
			for (Arc a : arcs)
				couleurArcs.put(a, c);
		}

		Visualisation f = new Visualisation(title);
		f.couleurVilles(couleurVilles);
		f.couleurArcs(couleurArcs);
		f.definirGraphe(g);
	}
		
	// Q5: Kruskal all components together
	public static void test5(){
		initFrance(4000);
		long t0 = System.currentTimeMillis();
		Collection<Arc> arcs = SpanningTree.kruskal(new UnionFind(), g);
		long tf = System.currentTimeMillis();
		System.out.println("Computation time (Kruskal first version) : " + (tf - t0) + " ms");
		analyzeMonoColor(arcs, true, "Kruskal first version");
	}

	// Q6: Kruskal separating the components
	public static void test6(){
		initFrance(4000);
		long t0 = System.currentTimeMillis();
		Collection<Collection<Arc>> listes = SpanningTree.kruskal(g);      
		long tf = System.currentTimeMillis();
		System.out.println("Computation time (Kruskal second version) : " + (tf - t0) + " ms");
		analyzeMultiColor(listes, true, "Kruskal second version");
	}
	
	// Q7: Prim tree
	public static void test7(){
		initFrance(4000);
		HashSet<Ville> nonVisited=new HashSet<Ville>();
		for (Ville v:g.villes()) nonVisited.add(v);
		long t0 = System.currentTimeMillis();
		Ville start=carte.premiereVille("Palaiseau");
		Collection<Arc> arcs = SpanningTree.primTree(nonVisited,start, g);
		long tf = System.currentTimeMillis();
		System.out.println("Computation time (Prim first version) : " + (tf - t0) + " ms");
		analyzeMonoColor(arcs, true, "Prim first version");
	}

	// Q8: Prim forest
	public static void test8(){
		initFrance(4000);
		long t0 = System.currentTimeMillis();
		Collection<Collection<Arc>> listes = SpanningTree.primForest(g);
		long tf = System.currentTimeMillis();
		System.out.println("Computation time (Prim second version) : " + (tf - t0) + " ms");
		analyzeMultiColor(listes, true, "Prim second version");
	}

	public static void main(String[] args) {
		//test1();
		//test2();
		//test3();
		//test4();
		//test5();
		test6();
		//test7();
		//test8();
	}
}









